﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace SeriTile.GUI
{
    public class GUISystem
    {
        public GraphicsDevice GraphicsDevice { get; private set; }
        public ITile DefaultCursor { get; set; }

        public GUIControl RootControl;

        private SpriteBatch m_spriteBatch;

        public GUIControl MouseOverControl { get; private set; }
        public SoundEffect CommonClickSound { get; set; }

        private MouseState m_previousMouseState;

        public GUISystem(GraphicsDevice graphicsDevice)
        {
            GraphicsDevice = graphicsDevice;
            m_spriteBatch = new SpriteBatch(graphicsDevice);
        }

        public void Draw()
        {
            m_spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend);
            RootControl.Draw(m_spriteBatch);

            MouseState mouseState = Mouse.GetState();
            if(MouseOverControl != null && MouseOverControl.Cursor!= null)
                MouseOverControl.Cursor.Draw(m_spriteBatch, new Vector2(mouseState.X, mouseState.Y));
            else if (DefaultCursor != null)
                DefaultCursor.Draw(m_spriteBatch, new Vector2(mouseState.X, mouseState.Y));
            m_spriteBatch.End();

        }

        public void Update(float deltaTime)
        {
            RootControl.Update(deltaTime);

            MouseState state = Mouse.GetState();

            GUIControl newMouseOverControl = RootControl.GetMouseOver(state.X, state.Y);
            if (MouseOverControl != newMouseOverControl)
            {
                if (MouseOverControl != null)
                    MouseOverControl.FireMouseOff();

                MouseOverControl = newMouseOverControl;

                if (MouseOverControl != null)
                    MouseOverControl.FireMouseOver();
            }

            if (state.LeftButton != m_previousMouseState.LeftButton)
            {
                if (state.LeftButton == ButtonState.Pressed)
                {
                    if (MouseOverControl != null)
                    {
                        if (CommonClickSound != null)
                            CommonClickSound.Play();
                        MouseOverControl.FireMouseDown();
                    }
                }
                else
                {
                    if (MouseOverControl != null)
                        MouseOverControl.FireMouseUp();
                }
            }


            m_previousMouseState = state;

        }

    }
}
