﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using Microsoft.Xna.Framework;
using SeriTile.GUI;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile.GUI
{
    public class TextControl : GUIControl
    {
        public string Text { set; get; }
        public SpriteFont Font { set; get; }
        public Color TextColor { set; get; }

        public TextControl(SpriteFont font, string text, Color color = default(Color)):base()
        {
            Font = font;
            Text = text;
            if (color == default(Color))
                color = Color.White;
            TextColor = color;

            Size = font.MeasureString(Text);
            MousePassThrough = true;
        }

        public void SizeToText()
        {
            Size = Font.MeasureString(Text);
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            if (!Visible)
                return;

            if (Disabled)
            {
                spriteBatch.DrawString(Font, Text, ScreenPosition, Color.LightGray);
            }
            else
            {
                spriteBatch.DrawString(Font, Text, ScreenPosition, TextColor);
            }


            base.Draw(spriteBatch);
        }

    }
}
