﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using System.IO;
using System.Xml;
using System.IO.Compression;

namespace SeriTile
{
    class ReaderHelper
    {
        public static BinaryReader GetDataReader(int width, int height, XmlReader childReader)
        {
            string encoding = childReader.GetAttribute("encoding");
            string compression = childReader.GetAttribute("compression");
            BinaryReader br = null;
            if (encoding == "base64")
            {
                int dataSize = (width * height *  sizeof(Int32));
                byte[] buffer = new byte[dataSize];
                string content = childReader.ReadElementContentAsString().Trim();
                buffer = Convert.FromBase64String(content);

                Stream stream;
                Stream memoryStream = new MemoryStream(buffer, true);

                if (compression == "gzip")
                {
                    stream = new GZipStream(memoryStream, CompressionMode.Decompress);
                }
                else
                {
                    stream = memoryStream;
                }
                
                br = new BinaryReader(stream);
            }
            return br;
        }

        public static bool AtEndTagFor(XmlReader xmlReader, string elementName)
        {
            return (!xmlReader.IsStartElement() && xmlReader.Name == elementName);
        }
    }
}
