﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile
{
    public class StateManager:Game
    {
        protected GraphicsDeviceManager m_graphics;
        private IGameState m_state;
        private SpriteBatch m_spriteBatch;
        private SpriteFont m_font;

        public Color ClearColor { get; set; }
        public bool ShowFPS { get; set; }
        // Must be set before Run is called.
        public string FPSFontName { private get; set; }

        public StateManager(IGameState startingState, int bufferWidth, int bufferHeight)
        {
            m_graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            m_graphics.PreferredBackBufferWidth = bufferWidth;
            m_graphics.PreferredBackBufferHeight = bufferHeight;

            m_state = startingState;
        }

        protected override void LoadContent()
        {
            m_spriteBatch = new SpriteBatch(GraphicsDevice);
            if (!string.IsNullOrEmpty(FPSFontName))
            {
                try
                {
                    m_font = Content.Load<SpriteFont>(FPSFontName);
                }
                catch (System.Exception)
                {

                }
            }

            m_state.LoadContent(GraphicsDevice, Content);
        }

        protected override void Update(GameTime gameTime)
        {
            m_state.Update(gameTime);
            IGameState newState = m_state.GetNextState();
            if (newState != null)
            {
                m_state.UnloadContent();
                m_state = newState;
                m_state.LoadContent(GraphicsDevice, Content);
            }

            base.Update(gameTime);
        }

        protected override void UnloadContent()
        {
            m_state.UnloadContent();
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            m_state.Draw(gameTime);

            if (ShowFPS && m_font != null)
            {
                m_spriteBatch.Begin();
                m_spriteBatch.DrawString(m_font, String.Format("{0:0.0} FPS", 1.0f / gameTime.ElapsedGameTime.TotalSeconds), new Vector2(0, 0), Color.White);
                m_spriteBatch.End();
            }

            base.Draw(gameTime);
        }

        protected override void Initialize()
        {


            base.Initialize();
        }

    }
}
