﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using System.IO;
using System.Xml;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace SeriTile
{
    public class Sprite
    {
        public System.Collections.Generic.Dictionary<string, AnimatedTile> m_animations = new System.Collections.Generic.Dictionary<string, AnimatedTile>();
        AnimatedTile m_animation;
        public delegate void OnAnimationFinishedHandler(Sprite sender, EventArgs eventArgs);
        public OnAnimationFinishedHandler OnAnimationFinished;

        public Sprite(string resourcePath, string fileName)
        {
            string xmlFilePath = resourcePath + fileName;
            using (FileStream fs = new FileStream(xmlFilePath, System.IO.FileMode.Open))
            {
                using (XmlReader xmlReader = XmlReader.Create(fs))
                {
                    xmlReader.ReadToFollowing("Sprite");
                    LoadSprite(xmlReader, resourcePath);
                }
            }
        }

        public Sprite(XmlReader xmlReader, string resourcePath)
        {
            LoadSprite(xmlReader, resourcePath);
        }

        private void LoadSprite(XmlReader xmlReader, string resourcePath)
        {
            string tilesetName = xmlReader.GetAttribute("TileSetName");
            string tilesetFile = xmlReader.GetAttribute("TileSet");
            TileSetManager tilesetManager = TileSetManager.GetSingleton();
            tilesetManager.LoadTileSet(tilesetName, resourcePath, tilesetFile);
            TileSet tileset = tilesetManager.GetTileSet(tilesetName);

            while (xmlReader.Read() && !ReaderHelper.AtEndTagFor(xmlReader, "TileSet"))
            {
                if(xmlReader.IsStartElement())
                {
                    if (xmlReader.Name == "Animation")
                    {
                        
                        ITile tile = null;
                        string tileName = xmlReader.GetAttribute("Tile");
                        if (string.IsNullOrEmpty(tileName))
                        {
                            int id = Int32.Parse(xmlReader.GetAttribute("ID"));
                            tile = tileset.getTile(id);
                        }
                        else
                        {
                            tile = tileset.getTile(tileName);
                        }

                        string animName = xmlReader.GetAttribute("Name");
                        addAnimation(animName, tile);
                    }
                }
            }
        }

        public AnimatedTile createAnimation(string name, bool loop)
        {
            AnimatedTile animation = new AnimatedTile(name, loop);
            m_animations.Add(name, animation);
            return animation;
        }

        public void addAnimation(string name, ITile tile)
        {
            AnimatedTile animation = new AnimatedTile(tile);
            animation.OnFinished += OnTileAnimationFinished;
            m_animations.Add(name, animation);
        }

        public void OnTileAnimationFinished(AnimatedTile tile, EventArgs args)
        {
            if(OnAnimationFinished != null)
                OnAnimationFinished(this, new EventArgs());
        }

        public void changeAnimation(string name)
        {
            if(OnAnimationFinished != null)
                OnAnimationFinished(this, new EventArgs());
            if (m_animation != m_animations[name] || m_animation.Finished)
            {
                m_animation = m_animations[name];
                m_animation.Start();
            }
        }

        public void update(float deltaTime)
        {
            m_animation.update(deltaTime);
        }

        public ITile Tile
        {
            get
            {
                return m_animation.Tile;
            }
        }

        public void Draw(SpriteBatch spriteBatch, Vector2 blitPoint)
        {
            Tile.Draw(spriteBatch, blitPoint);
        }
    }
}
