﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using System.Collections.Generic;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile
{
    class IsoDiamondTileLayer : TileLayer
    {
        public IsoDiamondTileLayer(int width, int height, int tileWidth, int tileHeight):
            base(width, height, tileWidth, tileHeight)
        {
        }

        public IsoDiamondTileLayer(int width, int height, int tileWidth, int tileHeight, XmlReader xmlReader, List<TilesetGID> tilesetGID) :
            base(width, height, tileWidth, tileHeight, xmlReader, tilesetGID)
        {
        }

        public override void Draw(SpriteBatch spriteBatch, Point drawOffset)
        {
            if (IsEmpty)
                return;
            Vector2 drawPoint;
            Vector2 blitPoint = new Vector2();

            for (int b = 0; b < m_tileCountY; b++)
            {
                drawPoint = TileToWorld(new TilePoint(0, b));
                drawPoint.X -= m_tileWidth / 2;
                drawPoint.Y -= m_tileHeight / 2;
                for (int a = 0; a < m_tileCountX; a++)
                {
                    if (IsValidTile(a, b))
                    {
                        ITile tile = m_tiles[a, b];
                        if (tile != null)
                        {
                            int offsetY = tile.Rectangle.Height - m_tileHeight;
                            blitPoint.X = drawPoint.X - drawOffset.X;
                            blitPoint.Y = drawPoint.Y - offsetY - drawOffset.Y;
                            tile.Draw(spriteBatch, blitPoint);
                        }

                    }
                    drawPoint.X += m_tileWidth / 2;
                    drawPoint.Y += m_tileHeight / 2;
                }
            }
 
        }

        public override void DrawEntities(SpriteBatch spriteBatch, Point drawOffset, TileMap.TileInfo[,] tileInfo)
        {
            for (int b = 0; b < m_tileCountY; b++)
            {
                for (int a = 0; a < m_tileCountX; a++)
                {
                    if (IsValidTile(a, b))
                    {
                        TileMap.TileInfo tile = tileInfo[a, b];
                        foreach (Entity entity in tile.Entities)
                        {
                            if (entity != null)
                            {
                                entity.Draw(spriteBatch);
                            }
                        }
                    }
                }
            }
        }

        public override Vector2 TileToWorld(TilePoint tilePoint)
        {
            Vector2 worldCoord = new Vector2();
            worldCoord.X = (tilePoint.X * m_tileWidth / 2) - (m_tileWidth / 2 * tilePoint.Y) + m_tileWidth / 2;
            worldCoord.Y = (tilePoint.Y * m_tileHeight / 2) + (m_tileHeight / 2 * tilePoint.X) + m_tileHeight / 2;
            return worldCoord;
        }

        public override TilePoint WorldToTile(Vector2 worldCoord)
        {
            int x = (int)Math.Round((double)(worldCoord.X / m_tileWidth + worldCoord.Y / m_tileHeight)) - 1;
            int y = (int)Math.Round((double)(-worldCoord.X / m_tileWidth + worldCoord.Y / m_tileHeight));
            return new TilePoint(x, y);
        }

    }


}
