﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System.Collections.Generic;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile
{
    class IsoStaggeredTileLayer:TileLayer
    {
        public IsoStaggeredTileLayer(int width, int height, int tileWidth, int tileHeight):
            base(width, height, tileWidth, tileHeight)
        {
        }

        public IsoStaggeredTileLayer(int width, int height, int tileWidth, int tileHeight, XmlReader xmlReader, List<TilesetGID> tilesetGID) :
            base(width, height, tileWidth, tileHeight, xmlReader, tilesetGID)
        {
        }

        public static ITile PickHelper { get; set; }

        public override void Draw(SpriteBatch spriteBatch, Point drawOffset)
        {
            if (IsEmpty)
                return;
            Vector2 blitPoint = new Vector2();

            for (int j = 0, b = 0; b < m_tileCountY; j += m_tileHeight / 2, b++)
            {
                for (int i = 0, a = 0; a < m_tileCountX; i += m_tileWidth, a++)
                {
                    int offsetX = (int)(b % 2 == 1 ? -m_tileWidth / 2 : 0);
                    if (IsValidTile(a, b))
                    {
                        ITile tile = m_tiles[a, b];
                        if (tile != null)
                        {
                            int offsetY = tile.Rectangle.Height - m_tileHeight;
                            blitPoint.X = i - offsetX - drawOffset.X;
                            blitPoint.Y = j - offsetY - drawOffset.Y;
                            tile.Draw(spriteBatch, blitPoint);
                        }
                    }
                }
            }
 
        }

        public override void DrawEntities(SpriteBatch spriteBatch, Point drawOffset, TileMap.TileInfo[,] tileInfo)
        {
            for (int j = 0, b = 0; b < m_tileCountY; j += m_tileHeight / 2, b++)
            {
                for (int i = 0, a = 0; a < m_tileCountX; i += m_tileWidth, a++)
                {
                    if (IsValidTile(a, b))
                    {
                        TileMap.TileInfo tile = tileInfo[a, b];
                        foreach (Entity entity in tile.Entities)
                        {
                            if (entity != null)
                            {
                                entity.Draw(spriteBatch);
                            }
                        }
                    }
                }
            }
        }

        public override Vector2 TileToWorld(TilePoint tilePoint)
        {
            Vector2 worldCoord = new Vector2();
            worldCoord.X = tilePoint.X * m_tileWidth + (tilePoint.Y % 2 == 1 ? m_tileWidth / 2 : 0) + m_tileWidth / 2;
            worldCoord.Y = tilePoint.Y * (m_tileHeight / 2) + m_tileHeight / 2; 
            
            return worldCoord;
        }

        /// If iso, PickHelper needs to be set up to a tile that looks like the red/yellow/green/blue tile at
        /// the bottom of http://www.gamedev.net/reference/articles/article747.asp
        public override TilePoint WorldToTile(Vector2 worldCoord)
        {
            SeriTile.TilePoint tileLoc = null;

            int x = (int)(worldCoord.X / m_tileWidth);
            int y = (int)(worldCoord.Y * 2 / m_tileHeight);
            int rx = (int)(worldCoord.X % m_tileWidth + PickHelper.Rectangle.X);
            int ry = (int)(worldCoord.Y % m_tileHeight + PickHelper.Rectangle.Y);
            uint[] pixels = new uint[PickHelper.Image.Width * PickHelper.Image.Height];
            PickHelper.Image.GetData(pixels);
            uint pixel = pixels[(int)(rx + ry * PickHelper.Image.Width)];
            uint alpha = (pixel >> 24) & 0xFF;
            uint red = (pixel >> 16) & 0xFF;
            uint green = (pixel >> 8) & 0xFF;
            uint blue = (pixel) & 0xFF;
            if (alpha == 0)
            {
                tileLoc = new SeriTile.TilePoint(x, y);
            }
            else if (red > 0)
            {
                if (green > 0)//Yellow
                {
                    tileLoc = new SeriTile.TilePoint(x, y - 1);
                }
                else//Red
                {
                    tileLoc = new SeriTile.TilePoint(x - 1, y - 1);
                }
            }
            else if (green > 0)//Green
            {
                tileLoc = new SeriTile.TilePoint(x - 1, y + 1);
            }
            else if (blue > 0)//Blue
            {
                tileLoc = new SeriTile.TilePoint(x, y + 1);
            }

            return tileLoc;
        }
    }
}
