﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System.Collections.Generic;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile
{
    class OrthoTileLayer:TileLayer
    {
        public OrthoTileLayer(int width, int height, int tileWidth, int tileHeight):
            base(width, height, tileWidth, tileHeight)
        {
        }

        public OrthoTileLayer(int width, int height, int tileWidth, int tileHeight, XmlReader xmlReader, List<TilesetGID> tilesetGID) :
            base(width, height, tileWidth, tileHeight, xmlReader, tilesetGID)
        {
        }

        public override void Draw(SpriteBatch spriteBatch, Point drawOffset)
        {
            if (IsEmpty)
                return;

            Vector2 blitPoint = new Vector2();
            int tileCountX = ScreenWidth / m_tileWidth;
            int tileCountY = ScreenHeight / m_tileHeight;
            Point offset = new Point(drawOffset.X % m_tileWidth, drawOffset.Y % m_tileHeight);

            TilePoint tilePos = new TilePoint(drawOffset, m_tileWidth);
            for (int j = 0, b = tilePos.Y; b < tilePos.Y + tileCountY + 2; j += m_tileHeight, b++)
            {
                for (int i = 0, a = tilePos.X; a < tilePos.X + tileCountX + 1; i += m_tileWidth, a++)
                {
                    if (IsValidTile(a,b))
                    {
                        ITile tile = m_tiles[a,b];
                        if (tile != null)
                        {
                            blitPoint.X = i - offset.X;
                            blitPoint.Y = j - offset.Y;
                            //Debug coloring
                            //Color testColor = tileInfo.Entity != null ? Colors.Yellow : (!tileInfo.Collision ? Colors.Red : Colors.White);
                            //tile.Blit(writeableBmp, blitPoint, testColor);
                            tile.Draw(spriteBatch, blitPoint);
                        }

                    }
                }
            }
        }

        public override Vector2 TileToWorld(TilePoint tilePoint)
        {
            return tilePoint.ToVector2(m_tileWidth);
        }

        public override TilePoint WorldToTile(Vector2 worldCoord)
        {
            return new SeriTile.TilePoint((int)(worldCoord.X / m_tileWidth), (int)(worldCoord.Y / m_tileHeight));
        }

        public override void DrawEntities(SpriteBatch spriteBatch, Point drawOffset, TileMap.TileInfo[,] tileInfo)
        {
            int tileCountX = ScreenWidth / m_tileWidth;
            int tileCountY = ScreenHeight / m_tileHeight;

            TilePoint tilePos = new TilePoint(drawOffset, m_tileWidth);
            for (int j = 0, b = tilePos.Y; b < tilePos.Y + tileCountY + 2; j += m_tileHeight, b++)
            {
                for (int i = 0, a = tilePos.X; a < tilePos.X + tileCountX + 1; i += m_tileWidth, a++)
                {
                    if (IsValidTile(a, b))
                    {
                        TileMap.TileInfo tile = tileInfo[a,b];
                        foreach (Entity entity in tile.Entities)
                        {
                            if (entity != null)
                            {
                                entity.Draw(spriteBatch);
                            }
                        }
                    }
                }
            }
        }
    }
}
