﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile
{
    abstract class TileLayer
    {
        protected ITile[,] m_tiles;
        public bool IsEmpty { get; private set; }
        protected int ScreenWidth;
        protected int ScreenHeight;
        protected int m_tileWidth;
        protected int m_tileHeight;
        protected int m_tileCountX;
        protected int m_tileCountY;
        public bool DrawEntitiesAfter { get; set; }

        public TileLayer(int width, int height, int tileWidth, int tileHeight)
        {
            m_tiles = new ITile[width, height];
            IsEmpty = true;
            m_tileCountX = width;
            m_tileCountY = height;
            m_tileWidth = tileWidth;
            m_tileHeight = tileHeight;
        }

        public TileLayer(int width, int height, int tileWidth, int tileHeight, XmlReader xmlReader, List<TilesetGID> tilesetGID)
        {
            m_tiles = new ITile[width, height];
            IsEmpty = true;
            m_tileCountX = width;
            m_tileCountY = height;
            m_tileWidth = tileWidth;
            m_tileHeight = tileHeight;

            using (BinaryReader br = ReaderHelper.GetDataReader(width, height, xmlReader))
            {
                ITile tile = null;
                for (int i = 0; i < height; i++)
                {
                    for (int j = 0; j < width; j++)
                    {
                        Int32 tileIndex = br.ReadInt32();
                        if (tileIndex > 0)
                        {
                            IsEmpty = false;
                            foreach (TilesetGID tilesetGid in tilesetGID)
                            {
                                if (tileIndex <= tilesetGid.MaxGID)
                                {
                                    tile = tilesetGid.GetTileFromGID(tileIndex);
                                    break;
                                }
                            }
                            m_tiles[j, i] = tile;
                        }
                    }
                }
            }

        }

        public void SetBounds(int screenWidth, int screenHeight)
        {
            ScreenWidth = screenWidth;
            ScreenHeight = screenHeight;
        }

        public bool IsValidTile(int x, int y)
        {
            return (x >= 0 && y >= 0 && y < m_tileCountY && x < m_tileCountX);
        }

        public abstract void Draw(SpriteBatch spriteBatch, Point drawOffset);
        public abstract void DrawEntities(SpriteBatch spriteBatch, Point drawOffset, TileMap.TileInfo[,] tileInfo);
        public abstract Vector2 TileToWorld(TilePoint isometricCoord);
        public abstract TilePoint WorldToTile(Vector2 worldCoord);

    }
}
