﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using Microsoft.Xna.Framework.Content;

namespace SeriTile
{
    public class MapManager
    {
        public int CacheCount { get; private set; }
        System.Collections.Generic.Dictionary<string, TileMap> m_maps = new System.Collections.Generic.Dictionary<string, TileMap>();
        private ContentManager m_content;
        public TileSetManager TileSetManager { get; private set; }
        private int m_screenWidth;
        private int m_screenHeight;
        private Entity m_player;
        public TileMap CurrentMap {get; private set;}
        public TileMap.TileEventHandler OnTMXEvent;

        public MapManager(ContentManager content, int screenWidth, int screenHeight)
        {
            m_content = content;
            CacheCount = 5;
            m_screenHeight = screenHeight;
            m_screenWidth = screenWidth;

            TileSetManager = new TileSetManager(content);
        }

        public Entity Player 
        { 
            get
            {
                return m_player;
            }

            set
            {
                m_player = value;
                if (CurrentMap != null)
                {
                    m_player.Map = CurrentMap;
                    m_player.UpdateCamera(m_screenWidth, m_screenHeight);
                }
            }
        }

        public TileMap GetMap(string fileName)
        {
            TileMap tileMap = null;
            if (m_maps.ContainsKey(fileName))
            {
                tileMap = m_maps[fileName];
            }
            else
            {
                if (TileSetManager == null)
                {
                    TileSetManager = new TileSetManager(m_content);
                }

                tileMap = new TileMap(m_content.RootDirectory+@"\", fileName, TileSetManager, m_screenWidth, m_screenHeight);
                tileMap.OnTransitionEvent += new TileMap.TileEventHandler(DoTransition);
                tileMap.OnTMXEvent += new TileMap.TileEventHandler(PropogateGenericEvent);
                m_maps.Add(fileName, tileMap);
            }
            tileMap.LastAccessed = DateTime.Now;

            while (m_maps.Count > CacheCount)
            {
                TileMap oldestMap = null;
                string oldestMapName = "";
                foreach (System.Collections.Generic.KeyValuePair<string, TileMap> map in m_maps)
                {
                    if (oldestMap == null)
                        oldestMap = map.Value;
                    else
                    {
                        if (map.Value.LastAccessed < oldestMap.LastAccessed)
                        {
                            oldestMap = map.Value;
                            oldestMapName = map.Key;
                        }
                    }
                }
                if (oldestMap != null)
                    m_maps.Remove(oldestMapName);
            }

            return tileMap;        
        }

        public void SetCurrentMap(TileMap map, string location)
        {
            if(m_player != null)
            {
                CurrentMap = map;
                m_player.Map = CurrentMap;
                m_player.TileLocation = CurrentMap.getTag(location).Point;
                m_player.UpdateCamera(m_screenWidth, m_screenHeight);
            }
        }

        public void SetCurrentMap(string mapName, string location)
        {
            TileMap map = GetMap(mapName);
            SetCurrentMap(map, location);
        }

        public void DoTransition(TileMap.TileInfo sender, EventArgs TileEventArgs)
        {
            string location;
            string target;
            bool hasLocation = sender.tag.Data.TryGetValue("Location", out location);
            bool hasTarget = sender.tag.Data.TryGetValue("Target", out target);

            if(hasLocation && hasTarget)
            {
                TileMap map = GetMap(target);
                SetCurrentMap(map, location);
            }
        }

        public void PropogateGenericEvent(TileMap.TileInfo sender, EventArgs TileEventArgs)
        {
            if (OnTMXEvent != null)
                OnTMXEvent(sender, TileEventArgs);
        }

    }
}
