﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using System;
using Microsoft.Xna.Framework;

namespace SeriTile
{
    /// <summary>
    /// Initializes the position at a certain radius around the emitter
    /// </summary>
    public class StartingRadiusInitializer : IParticleInitializer
    {
        public float MinRadius { get; set; }
        public float MaxRadius { get; set; }
        public Random m_random = new Random();

        public StartingRadiusInitializer(float minRadius, float maxRadius)
        {
            MinRadius = Math.Min(minRadius, maxRadius);
            MaxRadius = Math.Max(minRadius, maxRadius);
        }

        public void Initialize(Particle particle)
        {
            float Radius = (float)m_random.NextDouble() * (MaxRadius - MinRadius) + MinRadius;
            particle.Position = new Vector2(Radius,0);
        }

        public void Update(float deltaTime)
        {
        }
    }
}
