﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile
{
    public class Particle
    {
        public ITile Tile { get; set; }
        public Vector2 Position { get; set; }
        public float Speed { get; set; }
        public Vector2 Direction { get; set; }
        public Vector2 Acceleration { get; set; }
        public float Angle { get; set; }
        public float AngularVelocity { get; set; }
        public float Size {get; set;}
        public float SizeChange { get; set; }
        public float TTL { get; private set; }
        public bool IsAlive { get { return TTL > 0; } }

        private float m_maxTTL;
        private Vector4 m_color;
        private Vector4 m_colorChange;
        private Color m_renderColor;

        public Particle()
        {

        }

        public Vector2 Velocity 
        { 
            get
            {
                return Direction * Speed;
            }
            set
            {
                Speed = value.Length();
                Direction = value / Speed;
            }
        } 

        public void Init(ITile tile,
            Vector2 position,
            Vector2 velocity,
            float angle,
            float angularVelocity,
            Color colorBegin,
            Color colorEnd,
            float size = 1.0f,
            float sizeChange = 0,
            float ttl = 1)
        {
            Tile = tile;
            Position = position;
            Velocity = velocity;
            Acceleration = new Vector2(0,0);
            Angle = angle;
            AngularVelocity = angularVelocity;
            m_color = colorBegin.ToVector4();
            m_colorChange = (colorEnd.ToVector4() - m_color) / ttl;
            m_renderColor = new Color(m_color);
            Size = size;
            SizeChange = sizeChange;
            TTL = ttl;
            m_maxTTL = ttl;
        }

        public void Kill()
        {
            TTL = 0;
        }

        public void Update(float deltaTime)
        {
            TTL -= deltaTime;
            Velocity += (Acceleration * deltaTime);
            Position += (Velocity * deltaTime);
            Angle += AngularVelocity * deltaTime;
            m_color += m_colorChange * deltaTime;
            m_renderColor = new Color(m_color);
            Size += SizeChange * deltaTime;
        }

        public void Draw(SpriteBatch spriteBatch, Vector2 drawOffset)
        {
            if(!IsAlive)
                return;
            
            Vector2 origin = new Vector2(Tile.Rectangle.Left + Tile.Rectangle.Width / 2, Tile.Rectangle.Top + Tile.Rectangle.Height / 2);

            spriteBatch.Draw(Tile.Image, Position + drawOffset, Tile.Rectangle, m_renderColor, Angle, origin, Size, SpriteEffects.None, 0);
        }

    }
}
