﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using Microsoft.Xna.Framework;

namespace SeriTile
{
    public class TilePoint
    {
        public int X { get; set; }
        public int Y { get; set; }
        public TilePoint(int x, int y)
        {
            X = x;
            Y = y;
        }

        public TilePoint(Point point, int tileSize)
        {
            X = (int)(point.X / tileSize);
            Y = (int)(point.Y / tileSize);
        }

        private TilePoint()
        {

        }

        public Vector2 ToVector2(int tileSize)
        {
            return new Vector2((int)((X + .5) * tileSize), (int)((Y + .5) * tileSize));
        }

        public static bool operator ==(TilePoint a, TilePoint b)
        {
            // If both are null, or both are same instance, return true.
            if (System.Object.ReferenceEquals(a, b))
            {
                return true;
            }

            // If one is null, but not both, return false.
            if (((object)a == null) || ((object)b == null))
            {
                return false;
            }

            // Return true if the fields match:
            return a.X == b.X && a.Y == b.Y;
        }

        public static bool operator !=(TilePoint a, TilePoint b)
        {
            return !(a == b);
        }

        public override bool Equals(object obj)
        {
            return Equals(obj as TilePoint);
        }
        public bool Equals(TilePoint obj)
        {
            return obj == this;
        }


        public override int GetHashCode()
        {
            return X * 15000 + Y;
        }


    }
}
