﻿/*
-----------------------------------------------------------------------------
Copyright (c) 2009-2012 Sara Kempen

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
-----------------------------------------------------------------------------
*/

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace SeriTile
{
    public class TintedTile: ITile
    {
        ITile m_baseTile;
        Color m_tint;

        public TintedTile(ITile baseTile, Color tint)
        {
            m_baseTile = baseTile;
            m_tint = tint;
        }

        public Rectangle Rectangle
        {
            get
            {
                return m_baseTile.Rectangle;
            }
        }

        public Point Offset 
        {
            get
            {
                return m_baseTile.Offset;
            }
        }

        public Texture2D Image 
        {
            get
            {
                return m_baseTile.Image;
            }
        }

        public void Draw(SpriteBatch spriteBatch, Vector2 position, Color? color = null, float rotation = 0, Vector2 origin = default(Vector2), float scale = 1, SpriteEffects effects = SpriteEffects.None, float layerDepth = 0)
        {
            Color realColor;
            if (color.HasValue)
                realColor = color.Value;
            else
                realColor = m_tint;
            spriteBatch.Draw(Image, position, Rectangle, realColor, rotation, origin, scale, effects, layerDepth);
        }

        public void Draw(SpriteBatch spriteBatch, Vector2 position)
        {
            spriteBatch.Draw(Image, position, Rectangle, m_tint);
        }
    }
}
